@@ -27,10 +27,13 @@
 
 #include <config.h>
 
+#include <glib/gmessages.h>
+#include <glib/gstrfuncs.h>
+#include <libgnomevfs/gnome-vfs-init.h>
+#include <libgnomevfs/gnome-vfs-private-utils.h>
+#include <libgnomevfs/gnome-vfs-utils.h>
 #include <stdlib.h>
 #include <string.h>
-#include "gnome-vfs.h"
-#include "gnome-vfs-private-utils.h"
 
 #define TEST_ASSERT(expression, message) \
 	G_STMT_START { if (!(expression)) test_failed message; } G_STMT_END
@@ -202,6 +205,29 @@ test_uri_has_fragment_id (const char *input,
 	gnome_vfs_uri_unref (uri);
 }
 
+static void
+test_uri_extract_dirname (const char *input,
+			  const char *expected_output)
+{
+	GnomeVFSURI *uri;
+	char *output;
+
+	uri = gnome_vfs_uri_new (input);
+	if (uri == NULL) {
+		output = g_strdup ("NULL");
+	} else {
+		output = gnome_vfs_uri_extract_dirname (uri);
+	}
+
+	if (strcmp (output, expected_output) != 0) {
+		test_failed ("test_uri_extract_dirname (%s) resulted in %s instead of %s",
+			     input, output, expected_output);
+	}
+
+	g_free (output);
+	gnome_vfs_uri_unref (uri);
+}
+
 static void
 test_uri_parent (const char *input,
 		 const char *expected_output)
@@ -362,7 +388,6 @@ main (int argc, char **argv)
 	make_asserts_break ("GnomeVFS");
 
 	/* Initialize the libraries we use. */
-	g_thread_init (NULL);
 	gnome_vfs_init ();
 
 	/* Test the "make canonical" call for pathnames. */
@@ -583,6 +608,11 @@ main (int argc, char **argv)
 	test_uri_has_fragment_id ("/tmp/#junk", "junk");
 	test_uri_has_fragment_id ("/tmp/#junk#", "junk#");
 
+	/* Test gnome_vfs_uri_extract_dirname (). */
+	test_uri_extract_dirname ("/", "/");
+	test_uri_extract_dirname ("/usr", "/");
+	test_uri_extract_dirname ("/usr/bin", "/usr");
+
 	/* test a escaping->unescaping round trip for funny characters */
 	test_file_path_to_uri_string ("/tmp/#backup_file#", "file:///tmp/#backup_file#", GNOME_VFS_URI_HIDE_NONE);
 	test_file_path_to_uri_string ("/tmp/percent%percent", "file:///tmp/percent%percent", GNOME_VFS_URI_HIDE_NONE);
@@ -617,8 +647,8 @@ main (int argc, char **argv)
 	test_uri_to_string ("glorp:", "NULL", GNOME_VFS_URI_HIDE_NONE);
 	test_uri_parent ("glorp:", "URI NULL");
 
-	test_uri_to_string ("file:", "file:", GNOME_VFS_URI_HIDE_NONE);
-	test_uri_to_string ("http:", "http:", GNOME_VFS_URI_HIDE_NONE);
+	test_uri_to_string ("file:", "file:///", GNOME_VFS_URI_HIDE_NONE);
+	test_uri_to_string ("http:", "http:///", GNOME_VFS_URI_HIDE_NONE);
 	test_uri_to_string ("file:/", "file:///", GNOME_VFS_URI_HIDE_NONE);
 
 	/* FIXME bugzilla.eazel.com 6022: At least for http, we don't
